const semver = require('semver');
const { merge } = require('webpack-merge');
const { LOG_PREFIX } = require('../helpers/constant');
const ProgressBarPlugin = require('progress-bar-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const HandleModuleLoader = require('./handleModuleLoader');
const fs = require('fs');
const chalk = require('chalk');
const { scriptVersion, appStatic, existsStatic, dotenv } = require('../scripts/utils/paths');
const overrides = require('../config-overrides');
const scriptPkg = require(`${scriptVersion}/package.json`);

// CRA 2.1.2 switched to using a webpack config factory
// https://github.com/facebook/create-react-app/pull/5722
// https://github.com/facebook/create-react-app/releases/tag/v2.1.2
const isWebpackFactory = semver.gte(scriptPkg && scriptPkg.version, '2.1.2');
const webpackFactoryEnvSuffix = process.env.NODE_ENV === 'production' ? '.prod' : '.dev';

// 展开自定义env变量
if (process.env.REACT_APP_ENV) {
  const custom_env = `${dotenv}.${process.env.REACT_APP_ENV}`;
  if (fs.existsSync(custom_env)) {
    require('dotenv-expand')(
      require('dotenv').config({
        path: custom_env,
      })
    );
  }
}

const webpackConfigPath = `${scriptVersion}/config/webpack.config${!isWebpackFactory ? webpackFactoryEnvSuffix : ''}`;
const webpackConfig = require(webpackConfigPath);

// override config in memory
let overrideConfig = isWebpackFactory ? (env) => overrides.webpack(webpackConfig(env), env) : overrides.webpack(webpackConfig, process.env.NODE_ENV);
// mjd config in memory
const newWebpackConfig = overrideConfig(process.env.NODE_ENV);
// 加载进度条
newWebpackConfig.plugins.push(
  new ProgressBarPlugin({
    width: 50, // 默认20，进度格子数量即每个代表进度数，如果是20，那么一格就是5。
    format: chalk.blue.bold(LOG_PREFIX) + chalk.yellow('[:bar] ') + chalk.green.bold(':percent') + ' (:elapsed秒)',
    stream: process.stderr, // 默认stderr，输出流
    complete: '=', // 默认“=”，完成字符
    clear: false, // 默认true，完成时清除栏的选项
    renderThrottle: '', // 默认16，更新之间的最短时间（以毫秒为单位）
  })
);
if (existsStatic) {
  // 复制static文件
  newWebpackConfig.plugins.push(
    new CopyWebpackPlugin({
      patterns: [
        {
          from: appStatic,
          to: 'static',
        },
      ],
    })
  );
}
newWebpackConfig.module = HandleModuleLoader(newWebpackConfig.module);
newWebpackConfig.stats = process.args.verbose ? 'verbose' : 'errors-warnings';
const { custom = {}, override } = overrides;
overrideConfig = merge(newWebpackConfig, custom);
overrideConfig = override ? override(overrideConfig) : overrideConfig;
require.cache[require.resolve(webpackConfigPath)].exports = () => overrideConfig;

module.exports = require(webpackConfigPath);
